@@ -22,3 +22,17 @@ FontMetric = function( fontfamily, fontsize ){
 	list( info = info, widths = widths )
 }
 
+#' @title Compute the width of a string
+#'
+#' @description Compute the width of a string. Internal use.
+#' 
+#' @param value string value
+#' @param fontfamily font name
+#' @param fontsize font size
+#' @param fontface font face
+#' @export 
+reporters_str_width = function( value, fontfamily, fontsize, fontface ){
+	fontMetric = .jnew(class.fontMetric, fontfamily, as.integer( fontsize ) )
+	width = .jcall( fontMetric, "I", "getWidth", as.integer( fontface ), value )
+	list( width = as.integer(width) )
+}
