@@ -48,6 +48,7 @@ import org.lysis.reporters.formats.ParProperties;
 import org.lysis.reporters.img.DrawingML;
 import org.lysis.reporters.lists.NumberingDefinition;
 import org.lysis.reporters.tables.FlexTable;
+import org.lysis.reporters.text.CodeBlock;
 import org.lysis.reporters.text.ParagraphSet;
 import org.lysis.reporters.text.RScript;
 import org.lysis.reporters.text.TOC;
@@ -99,7 +100,12 @@ public class docx4R {
 		Styles styles = basedoc.getMainDocumentPart().getStyleDefinitionsPart().getJaxbElement();      
 		for ( org.docx4j.wml.Style s : styles.getStyle() ) {			
 			if( s.getType().equals("paragraph") ){
-				styleDefinitions.put( s.getStyleId(), s.getName().getVal() );
+				try{
+					styleDefinitions.put( s.getAliases().getVal(), s.getName().getVal() );
+				} catch( Exception e){
+					styleDefinitions.put( s.getStyleId(), s.getName().getVal() );
+				}
+				
 			}
 		}
 	}
@@ -398,6 +404,39 @@ public class docx4R {
 		} else throw new Exception("can't find bookmark '" +bookmark+"'." );
 	}
 
+	public void add( CodeBlock script, ParProperties pp ) throws Exception{
+		
+		script.setDOCXMLPackage(basedoc);
+		script.setParProperties(pp);
+		script.setUID(eltIndex + 1 );
+		basedoc.getMainDocumentPart().getContent().addAll(script.get_docx_elt());
+		eltIndex = (int) script.nextUniqueId();
+		eventManage();
+	}
+	
+	
+	public void add( CodeBlock script, ParProperties pp, String bookmark ) throws Exception{
+		BookmarkObject bo = DocExplorer.getBookmarkObject(bookmark, basedoc);
+
+		if( bo.exists() ){
+		
+			script.set_docx_bookmark(bookmark, bo.getBookmarkID());
+			script.setDOCXMLPackage(basedoc);
+			script.setParProperties(pp);
+			script.setUID(eltIndex+1);
+
+			P p = bo.getP();
+		    int i = ((ContentAccessor)p.getParent()).getContent().indexOf(p);
+		    ((ContentAccessor)p.getParent()).getContent().remove(i);
+		    Vector<P> pars = script.get_docx_elt();
+		    for( int pid = 0 ; pid < script.size() ; pid++ ){
+		    	((ContentAccessor)p.getParent()).getContent().add(i+pid, pars.get(pid));
+		    }
+			eltIndex = eltIndex + script.size() ;
+			
+		} else throw new Exception("can't find bookmark '" +bookmark+"'." );
+	}
+
 
 	public void restartNumbering(){
 		if( Debug.debug_list ) System.err.println("restartNumbering: " + ordered_num_id );
@@ -544,26 +583,45 @@ public class docx4R {
 	}
 	
 
+//    public void insertExternalDocx( String file) throws Exception {
+//    		eltIndex = eltIndex+1;
+//    		InputStream s2 = new FileInputStream(new File(file));
+//    		String CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
+//    		MainDocumentPart main = basedoc.getMainDocumentPart();
+//			byte[] bytes = IOUtils.toByteArray(s2);
+//			s2.close();
+//    		AlternativeFormatInputPart afiPart = new AlternativeFormatInputPart(new PartName("/part" + 
+//    			(eltIndex) + ".docx"));
+//            afiPart.setContentType(new ContentType(CONTENT_TYPE));
+//            afiPart.setBinaryData(bytes);
+//            Relationship altChunkRel = main.addTargetPart(afiPart);
+//
+//            CTAltChunk chunk = new CTAltChunk();
+//            chunk.setId(altChunkRel.getId());
+//
+//            main.addObject(chunk);
+//            eltIndex++;
+//    }
     public void insertExternalDocx( String file) throws Exception {
-    		eltIndex = eltIndex+1;
-    		InputStream s2 = new FileInputStream(new File(file));
-    		String CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
-    		MainDocumentPart main = basedoc.getMainDocumentPart();
-			byte[] bytes = IOUtils.toByteArray(s2);
-			s2.close();
-    		AlternativeFormatInputPart afiPart = new AlternativeFormatInputPart(new PartName("/part" + 
-    			(eltIndex) + ".docx"));
-            afiPart.setContentType(new ContentType(CONTENT_TYPE));
-            afiPart.setBinaryData(bytes);
-            Relationship altChunkRel = main.addTargetPart(afiPart);
 
-            CTAltChunk chunk = new CTAltChunk();
-            chunk.setId(altChunkRel.getId());
+		MainDocumentPart main = basedoc.getMainDocumentPart();
+		
+		AlternativeFormatInputPart afiPart = new AlternativeFormatInputPart(new PartName("/doc_" + 
+    			(eltIndex) + ".docx") );
+		afiPart.setBinaryData(
+				new FileInputStream(file) );
+		
+		afiPart.setContentType(new ContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml")); //docx
 
-            main.addObject(chunk);
-            eltIndex++;
-    }
-    
+		Relationship altChunkRel = main.addTargetPart(afiPart);
+		
+		CTAltChunk ac = new CTAltChunk();
+		ac.setId(altChunkRel.getId());
+
+		main.addObject(ac);
+		eltIndex++;
+
+}
     
     public void insertExternalHTML( String file) throws Exception {
 		eltIndex = eltIndex+1;
@@ -571,7 +629,7 @@ public class docx4R {
 		String CONTENT_TYPE = "text/html";
 		MainDocumentPart main = basedoc.getMainDocumentPart();
 		byte[] bytes = IOUtils.toByteArray(s2);
-		AlternativeFormatInputPart afiPart = new AlternativeFormatInputPart(new PartName("/part" + 
+		AlternativeFormatInputPart afiPart = new AlternativeFormatInputPart(new PartName("/doc_" + 
 			(eltIndex) + ".html"));
         afiPart.setContentType(new ContentType(CONTENT_TYPE));
         afiPart.setBinaryData(bytes);
