@@ -61,7 +61,10 @@ public class BorderProperties {
 		    else if( borderStyle.equals("dashed") )
 		    	border.setVal(org.docx4j.wml.STBorder.DASHED);
 		    else border.setVal(org.docx4j.wml.STBorder.SINGLE);
-		} else border.setVal(org.docx4j.wml.STBorder.NONE);
+		} else {
+			border.setVal(org.docx4j.wml.STBorder.NONE);
+			border.setSz(BigInteger.valueOf(0));
+		}
 	    return border;
 	}
 	
@@ -92,6 +95,10 @@ public class BorderProperties {
 	
 	public String get_html_bd (String side) throws Exception {
 		String out = "";
+		
+		if( borderWidth < 1 ) return "";
+		if( borderStyle.equals("none") ) return "";
+		
 		String prefix = "border-" + side + "-";
 		out = out.concat(prefix);
 		out = out.concat("color:");
@@ -114,4 +121,6 @@ public class BorderProperties {
 
 		return out;
 	}
+	
+
 }
