@@ -33,6 +33,11 @@ public class FontMetric {
 		FontMetrics fm = c.getFontMetrics(fonts[face]);
 		return fm.getWidths();
 	}
+	public int getWidth(int face, String val){
+		Canvas c = new Canvas();
+		FontMetrics fm = c.getFontMetrics(fonts[face]);
+		return fm.stringWidth(val);
+	}
 	
 	public int[] getStr(int face){
 		Canvas c = new Canvas();
@@ -49,4 +54,16 @@ public class FontMetric {
 		Font ft = new Font( fontfamily, 0, 12 );
 		return ft.getFamily();
 	}
+	
+//	public static void main(String[] args){
+//		Font font = new Font("Arial",Font.PLAIN, 12);
+//	
+//		Canvas c = new Canvas();
+//		FontMetrics fm = c.getFontMetrics(font);
+//		int[] out = new int[109384];
+//		for(int i = 0 ; i < 512 ; i++ )
+//			out[i] = fm.charWidth(i);
+////			System.out.println(i + ":" + fm.charWidth(i));
+//
+//	}
 }
