@@ -46,7 +46,7 @@ public class ParagraphSet extends DocComponent implements HTML4R, PPTX4R, DOCX4R
 	            + "</p:nvPr>"
 		    + "</p:nvSpPr>"
 		    + "<p:spPr>"
-		      + "<a:xfrm>"
+		      + "<a:xfrm rot=\"${rot}\">"
 		        + "<a:off x=\"${offx}\" y=\"${offy}\"/>"
 		        + "<a:ext cx=\"${cx}\" cy=\"${cy}\"/>"
 		      + "</a:xfrm>"
@@ -168,6 +168,7 @@ public class ParagraphSet extends DocComponent implements HTML4R, PPTX4R, DOCX4R
         mappings.put("cy", "" + getCy() );
         mappings.put("offx", "" + getOffx() );
         mappings.put("offy", "" + getOffy() );
+        mappings.put("rot", "" + getRot() );
 
         Shape o = (Shape) XmlUtils.unmarshallFromTemplate(SAMPLE_SHAPE_START + "<a:p/>" + SAMPLE_SHAPE_END, mappings,Context.jcPML, Shape.class);
         return o;
