@@ -48,6 +48,7 @@ public class DocComponent {
 	private long cy;
 	private long offx;
 	private long offy;
+	private int rotate;
 	
 	private ParProperties parProp;
 	private PPr inherited_ppr;
@@ -68,6 +69,7 @@ public class DocComponent {
 		cy = -1;
 		offx = -1;
 		offy = -1;
+		rotate = 0;
 		parProp = null;
 		inherited_ppr = null;
 		docx_mlp = null;
@@ -88,7 +90,9 @@ public class DocComponent {
 	public long getOffy() {
 		return offy;
 	}
-	
+	public int getRot() {
+		return rotate;
+	}
 	public boolean validatePPTXComponent(){
 		if( offx < 0 ) return false;
 		if( cx < 0 ) return false;
@@ -158,6 +162,9 @@ public class DocComponent {
 	public void setOffy(long y){
 		offy = y;
 	}
+	public void setRot(int rot){
+		rotate = rot;
+	}
 	
 	public void setOffAndExt(long x, long y, long w, long h){
 		cx = w;
@@ -241,6 +248,7 @@ public class DocComponent {
 		xfrm.setOff(off);
 		off.setX(offx);
 		off.setY(offy);
+		xfrm.setRot(rotate);
 		return xfrm;
 	}
 	
@@ -265,10 +273,16 @@ public class DocComponent {
 		offx = xfrm.getOff().getX();
 		offy = xfrm.getOff().getY();
 		cx = xfrm.getExt().getCx();
-		cy = xfrm.getExt().getCy();		
+		cy = xfrm.getExt().getCy();
+        try{
+        	rotate = xfrm.getRot();
+        } catch( Exception e) {
+        	rotate = 0;
+        }
+        
 		if(Debug.debug) System.err.println("setXfrm > offx " + offx + 
 				" offy " + offy + " cx " + cx + 
-				" cy " + cy );
+				" cy " + cy + " rotate " + rotate );
 	}
 	
 	public long nextUniqueId(){
