@@ -49,6 +49,8 @@ static void PPTX_Polygon(int n, double *x, double *y, const pGEcontext gc,
 static void PPTX_Rect(double x0, double y0, double x1, double y1,
 		const pGEcontext gc, pDevDesc dev) ;
 
+static void PPTX_TextUTF8(double x, double y, const char *str, double rot,
+		double hadj, const pGEcontext gc, pDevDesc dev);
 static void PPTX_Text(double x, double y, const char *str, double rot,
 		double hadj, const pGEcontext gc, pDevDesc dev);
 
@@ -61,6 +63,7 @@ static void PPTX_MetricInfo(int c, const pGEcontext gc, double* ascent,
 static void PPTX_Size(double *left, double *right, double *bottom, double *top,
 		pDevDesc dev);
 
+static double PPTX_StrWidthUTF8(const char *str, const pGEcontext gc, pDevDesc dev);
 static double PPTX_StrWidth(const char *str, const pGEcontext gc, pDevDesc dev);
 
 SEXP R_PPTX_Device(SEXP filename, SEXP width, SEXP height, SEXP offx,
