@@ -24,18 +24,19 @@
 #include <R_ext/Boolean.h>
 
 
-double getFontSize(double cex, double fontsize, double lineheight);
+double getFontSize(double cex, double fontsize);
 void updateFontInfo(pDevDesc dev, R_GE_gcontext *gc);
 
 void DOC_MetricInfo(int c, const pGEcontext gc, double* ascent,
 		double* descent, double* width, pDevDesc dev);
 
+double DOC_StrWidthUTF8(const char *str, const pGEcontext gc, pDevDesc dev);
 double DOC_StrWidth(const char *str, const pGEcontext gc, pDevDesc dev);
-
+double translate_rotate_x(double x, double y, double rot, double h, double w, double hadj);
+double translate_rotate_y(double x, double y, double rot, double h, double w, double hadj);
 int get_and_increment_idx(pDevDesc dev);
-
 void register_element(pDevDesc dev);
-
+int getFontface( int ff );
 void closeFile( FILE *file);
 
 void DOC_ClipLine(double x0, double y0, double x1, double y1, pDevDesc dev);
